/*
 * Copyright (c) 2015, Majenko Technologies
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice, this
 *   list of conditions and the following disclaimer in the documentation and/or
 *   other materials provided with the distribution.
 *
 * * Neither the name of Majenko Technologies nor the names of its
 *   contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR
 * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
 * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON
 * ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */




// Requires a PRO license for ViSi-Genie Magic to operate.

#include <genieArduino.h>
#include <SoftwareSerial.h>

#define RESETLINE 4  // Change this if you are not using an Arduino Adaptor Shield Version 2 (see code below)

#define MFILE_READ 0
#define MFILE_WRITE 1
#define MFILE_APPEND 2
#define MFILE_ERASE 3
#define MFILE_DIR 4
#define MFILE_SCREEN_CAPTURE 5
#define MFILE_SIZE 6
#define NULL 0x00

Genie genie;

// Use software serial on pins 10 and 11, which means set the RX and TX jumpers on
// the Arduino breakout board to use D10 and D11.
SoftwareSerial serial(10, 11);

char charString[256]; //Array to hold bytes
boolean firstRunFlag = true;
boolean readDoneFlag = false;

//byte filename[] = {MFILE_READ, 'D', 'A', 'T', 'A','.', 'L', 'O', 'G', NULL};    // Example 1
//byte filename[] = {MFILE_READ, 'D','U','M','M','Y','.','T','X','T', NULL};      // Example 2
byte filename[] = {MFILE_READ, 'A', 'S', 'C', 'I','I','.', 'T','X','T', NULL};  // Example 3
byte filenameLength = sizeof(filename);

void setup(){    
  Serial.begin(9600);  // Console in Arduino IDE on Serial0
  serial.begin(9600);  
  genie.Begin(serial); // display connected to a software serial port

  //genie.AttachEventHandler(myGenieEventHandler); // Attach the user function Event Handler for processing events
  genie.AttachMagicByteReader(myGenieMagicHandler); // Attach the user function Magic Handler for processing Magic events
  // Reset the Display (change D4 to D2 if you have original 4D Arduino Adaptor)
  // THIS IS IMPORTANT AND CAN PREVENT OUT OF SYNC ISSUES, SLOW SPEED RESPONSE ETC
  // If NOT using a 4D Arduino Adaptor, digitalWrites must be reversed as Display Reset is Active Low, and
  // the 4D Arduino Adaptors invert this signal so must be Active High.
  pinMode(RESETLINE, OUTPUT);  // Set D4 on Arduino to Output (4D Arduino Adaptor V2 - Display Reset)
  digitalWrite(RESETLINE, 1);  // Reset the Display via D4
  delay(100);
  digitalWrite(RESETLINE, 0);  // unReset the Display via D4  
  delay (5000); //let the display start up after the reset (This is important)  

  Serial.println("Sending open and read request for the file: ");  
  for (int i = 1; i < filenameLength - 1; i++)
    Serial.print((char)filename[i]);    
  Serial.println();  
  //Open and read from <filename> on the microSD card in the Display
  genie.WriteMagicBytes(0, filename, filenameLength);  
}

void loop(){
  static uint32_t lastRun = 0;

  genie.DoEvents(); // This calls the library each loop to process the queued responses from the display  
  
  if (millis() > lastRun){
    if(firstRunFlag == true && readDoneFlag == true){
      Serial.println("File read done.");
      firstRunFlag = false;
    }    
    // do something else here
    lastRun = millis() + 5000; // Run this again in 5 seconds time
  }
}

void myGenieMagicHandler(uint8_t index, uint8_t length){
  // Library handles the 'Report Magic Bytes' and provides the Index and the Length of the incomming data to this function. 
  // The first byte of data to this function is the Function Code, which can be set from the WS4 application if the MagicObject 
  // Code can do more than 1 single thing. It is a way for the Arduino/Host to then identify what the data is exactly.
  
  byte FunctionCode;  
  byte MoreOrFinalByte;
  if (index == 0) // If the data is coming from MagicObject0
  {     
    if (length > 1) {             
      FunctionCode = genie.GetNextByte();      
      if (FunctionCode == MFILE_READ) // If the first byte indicated that this is a READ Function
      {
        MoreOrFinalByte = genie.GetNextByte(); // the second byte indicates if a message follows this one or not
        Serial.print("Received ");
        Serial.print(length, DEC);
        Serial.print(" bytes from magic object ");
        Serial.println(index, DEC);
        Serial.println("message:");
        for (int i = 0; i < length-2; i++){
          charString[i] = genie.GetNextByte(); // Get the bytes 1 by 1, and store in an array
        }
        for (int i = 0; i < length-2; i++){
          Serial.print(charString[i]);         // Print the array of characters to show what was received  
        }
        Serial.println();

        if(MoreOrFinalByte == 0xFF)            // the manual says, "More / Final byte - 0x00 indicates this is the last  
          Serial.println("Another message follows..."); //data for the file, 0xFF indicates another message follows this one."
        else{
          Serial.println("This is the last message.");
          readDoneFlag = true;
        }
        Serial.println(); 
      }
    }    
    // as per the manual, Only the ‘function code’ will be sent if an error occurs (eg file does not exist)
    else if (length == 1){ 
      FunctionCode = genie.GetNextByte();      
      if (FunctionCode == MFILE_READ) // If the first byte indicated this is a READ Function
      {     
        Serial.println("File does not exist!");
        Serial.println(); // Print a blank line (formatting only)
        readDoneFlag = true;
      }
    }
  }
}
